'use strict';
/* eslint-disable */

var _values = require('babel-runtime/core-js/object/values');

var _values2 = _interopRequireDefault(_values);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _entries = require('babel-runtime/core-js/object/entries');

var _entries2 = _interopRequireDefault(_entries);

var _layouter = require('../src/layouter.js');

var _layouter2 = _interopRequireDefault(_layouter);

var _imagesMock = require('./images-mock.js');

var _deepFreeze = require('deep-freeze');

var _deepFreeze2 = _interopRequireDefault(_deepFreeze);

var _chai = require('chai');

var _expectedOffsets = require('./expectedOffsets');

var _expectedOffsets2 = _interopRequireDefault(_expectedOffsets);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getItems = function getItems(count) {
  return (0, _deepFreeze2.default)(_imagesMock.testImages.slice(0, count));
};

var log = require('log-to-file');

describe('Expected Layouts', function () {

  var gallery = void 0;
  var layouter = void 0;
  var container = {};
  var styleParams = {};
  var styleParamsOptions = {};

  var getLayout = function getLayout(layoutParams) {
    return layouter.createLayout(layoutParams);
  };

  beforeEach(function () {

    var items = getItems();

    styleParamsOptions = {
      oneRow: [true, false],
      isVertical: [true, false],
      gallerySize: [100, 500],
      groupSize: [1, 3],
      groupTypes: ['1,2h,2v,3t,3b,3l,3r,3v,3h', '2h,2v,3t,3v,3h'],
      cubeImages: [true, false],
      cubeRatio: [1, 2],
      smartCrop: [true, false],
      chooseBestGroup: [true, false],
      collageDensity: [0, 1],
      minItemSize: [20, 200],
      imageMargin: [0, 100],
      fixedColumns: [0, 5]
    };

    container = {
      width: 1000,
      bounds: {
        visibleTop: 0,
        visibleBottom: 1000,
        renderedTop: 0,
        renderedBottom: 10000
      }
    };

    layouter = new _layouter2.default({ items: items, container: container, styleParams: styleParams });
  });

  it('should remain as expected', function () {

    if (process.env.IN_WALLABY) {
      return; //do not run this test in wallaby (takes too long)
    }

    var isDebug = false;

    var items = getItems(20);

    var totalOptions = [];
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = (0, _getIterator3.default)((0, _values2.default)(styleParamsOptions)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var option = _step.value;

        totalOptions.push((totalOptions[totalOptions.length - 1] || 1) * option.length);
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    var selectStyleParams = function selectStyleParams(seed) {
      var styleParams = {};
      var styleParamsArr = (0, _entries2.default)(styleParamsOptions);
      // console.log(styleParamsArr);
      var idx = 0;
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = (0, _getIterator3.default)(styleParamsArr), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var _ref = _step2.value;

          var _ref2 = (0, _slicedToArray3.default)(_ref, 2);

          var key = _ref2[0];
          var value = _ref2[1];

          styleParams[key] = value[Math.floor(seed / totalOptions[idx]) % value.length];
          idx++;
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      return styleParams;
    };

    var startTime = Date.now();
    for (var seed = 1; seed < totalOptions[totalOptions.length - 1]; seed++) {
      var _styleParams = selectStyleParams(seed);

      gallery = getLayout({ items: items, container: container, styleParams: _styleParams });

      var offsets = gallery.items.reduce(function (str, item) {
        var os = item.offset;
        var itemStr = '{t:' + Math.round(os.top) + ',l:' + Math.round(os.left) + ',r:' + Math.round(os.right) + ',b:' + Math.round(os.bottom) + '}';
        return str + itemStr;
      }, '');

      if (isDebug) {
        if (!(seed % 100)) {
          console.log('Completed ' + Math.round(100 * seed / totalOptions[totalOptions.length - 1]) + '% ' + seed + '/' + totalOptions[totalOptions.length - 1] + ' layouts (' + Math.floor((Date.now() - startTime) / 1000) + 's)');
        }
        log('\'' + offsets + '\',');
      } else {
        (0, _chai.expect)(offsets).to.equal(_expectedOffsets2.default[seed - 1]);
      }
    }
  });
});